use serde::Serialize;

use crate::services::artifacts_db::{AddCommitArtifact, GetLatestArtifact};
use crate::services::file_store::FileStore;

use super::archive::Archive;
use super::artifacts_db::ArtifactsDB;
use super::domain_events::DomainEvents;
use super::file_reader::FileReader;

use self::models::*;

pub struct ReleaseManager {
    archive: Archive,
    file_reader: FileReader,
    file_store: FileStore,
    domain_events: DomainEvents,
    artifacts_db: ArtifactsDB,
}

impl ReleaseManager {
    pub fn new(
        file_reader: FileReader,
        file_store: FileStore,
        archive: Archive,
        domain_events: DomainEvents,
        artifacts_db: ArtifactsDB,
    ) -> Self {
        Self {
            archive,
            file_reader,
            file_store,
            domain_events,
            artifacts_db,
        }
    }

    pub async fn upload_artifact(
        &self,
        request: UploadArtifact,
    ) -> anyhow::Result<UploadArtifactID> {
        let upload_id = uuid::Uuid::now_v7();

        self.file_store
            .upload_temp(upload_id.into(), request.file_path)
            .await?;

        Ok(upload_id.into())
    }

    pub async fn commit_artifact(&self, request: CommitArtifact) -> anyhow::Result<ArtifactID> {
        tracing::debug!("committing artifact: {:?}", request);
        let artifact_id = ArtifactID::new();

        let artifact = self.file_store.get_temp(request.upload_id).await?;

        self.file_store
            .upload_file(artifact_id.clone(), artifact)
            .await?;

        self.domain_events
            .publish_event(&serde_json::to_string(&CommittedArtifactEvent {
                artifact_id: artifact_id.to_string(),
            })?)
            .await?;

        self.artifacts_db
            .commit_artifact(AddCommitArtifact {
                app: request.app,
                branch: request.branch,
                artifact_id: artifact_id.clone().into(),
            })
            .await?;

        Ok(artifact_id)
    }

    pub async fn release(&self, release_req: Release) -> anyhow::Result<()> {
        tracing::debug!(
            app = release_req.app,
            branch = release_req.branch,
            cluster = release_req.cluster,
            "releasing latest commit"
        );

        let artifact = self
            .artifacts_db
            .get_latest_artifact(GetLatestArtifact {
                app: release_req.app,
                branch: release_req.branch,
            })
            .await?;

        tracing::trace!("found latest artifact: {:?}", artifact);

        Ok(())
    }
}

#[derive(Serialize)]
pub struct CommittedArtifactEvent {
    artifact_id: String,
}

pub mod extensions;
pub mod models;
